import 'dart:async';

import 'package:analyzer/dart/analysis/context_builder.dart';
import 'package:analyzer/dart/analysis/context_locator.dart';
import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/file_system/file_system.dart';
import 'package:analyzer/src/dart/analysis/driver.dart';
import 'package:analyzer_plugin/plugin/plugin.dart';
import 'package:analyzer/src/dart/analysis/driver_based_analysis_context.dart';
import 'package:analyzer_plugin/protocol/protocol_generated.dart' as plugin;
import 'package:analyzer_plugin/protocol/protocol_generated.dart';
import 'package:cat_analyzer_plugin/configs/rule_config.dart';
import 'package:cat_analyzer_plugin/model/error_issue.dart';
import 'package:cat_analyzer_plugin/utils/issue_map_util.dart';

class CustomCodeAnalysisPlugin extends ServerPlugin{
  CustomCodeAnalysisPlugin(ResourceProvider provider) : super(provider);

  var _filesFromSetPriorityFilesRequest = <String>[];

  // late YamlMap _yamlMap;
  //
  // String? _sourceUri;

  @override
  List<String> get fileGlobsToAnalyze => <String>['**/*.dart'];

  @override
  String get name => 'Custom Code Analysis';

  @override
  String get version => '1.0.0';

  // @override
  // Future<plugin.PluginVersionCheckResult> handlePluginVersionCheck(plugin.PluginVersionCheckParams parameters) {
  //   logUtil.info('parameters = $parameters');
  //   channel.sendNotification(plugin.PluginErrorParams(false, 'customErroor', parameters.toString()).toNotification());
  //   return super.handlePluginVersionCheck(parameters);
  // }

  @override
  void contentChanged(String path) {
    var driver = super.driverForPath(path)!;
    // logUtil.info('File content changed: $path');
    driver.addFile(path);
  }

  @override
  AnalysisDriverGeneric createAnalysisDriver(plugin.ContextRoot contextRoot) {
    final rootPath = contextRoot.root;
    final locator =
        ContextLocator(resourceProvider: resourceProvider).locateRoots(
      includedPaths: [rootPath],
      excludedPaths: contextRoot.exclude,
      optionsFile: contextRoot.optionsFile,
    );

    if (locator.isEmpty) {
      final error = StateError('Unexpected empty context');
      channel.sendNotification(plugin.PluginErrorParams(
        true,
        error.message,
        error.stackTrace.toString(),
      ).toNotification());
      throw error;
    }

    final builder = ContextBuilder(resourceProvider: resourceProvider);
    final context = builder.createContext(contextRoot: locator.first)
        as DriverBasedAnalysisContext;
    final dartDriver = context.driver;
    runZonedGuarded(
      () {
        dartDriver.results.listen((analysisResult) {
          _processResult(dartDriver, analysisResult);
        });
      },
      (e, stackTrace) {
        channel.sendNotification(
          plugin.PluginErrorParams(false, e.toString(), stackTrace.toString())
              .toNotification(),
        );
      },
    );
    return dartDriver;
  }

  void _onError(Object e, StackTrace stackTrace) {
    channel.sendNotification(
        plugin.PluginErrorParams(false, e.toString(), stackTrace.toString())
            .toNotification());
  }

  @override
  Future<plugin.AnalysisSetPriorityFilesResult> handleAnalysisSetPriorityFiles(
      plugin.AnalysisSetPriorityFilesParams parameters) async {
    _filesFromSetPriorityFilesRequest = parameters.files;
    _updatePriorityFiles();
    return plugin.AnalysisSetPriorityFilesResult();
  }

  void _updatePriorityFiles() {
    final filesToFullyResolve = {
      ..._filesFromSetPriorityFilesRequest,
      for (final driver2 in driverMap.values)
        ...(driver2 as AnalysisDriver).addedFiles,
    };
    final filesByDriver = <AnalysisDriverGeneric?, List<String>>{};
    for (final file in filesToFullyResolve) {
      final contextRoot = contextRootContaining(file);
      if (contextRoot != null) {
        final driver = driverMap[contextRoot];
        filesByDriver.putIfAbsent(driver, () => <String>[]).add(file);
      }
    }
    filesByDriver.forEach((driver, files) => driver!.priorityFiles = files);
  }

  // String? _getSourceUri(AnalysisDriver dartDriver) {
  //   String? _uri;
  //   // ignore: deprecated_member_use
  //   var _optionsFile = dartDriver.analysisContext.analysisOptions.optionsFilePath;
  //   if (_optionsFile != null) {
  //     // ignore: deprecated_member_use
  //     _uri = dartDriver.resourceProvider.pathContext.dirname(dartDriver.contextRoot!.optionsFilePath!);
  //   }
  //   return _uri;
  // }

  void _processResult(
      AnalysisDriver driver, ResolvedUnitResult analysisResult) {
    try {
      if (analysisResult.unit != null) {
        var _fullName = analysisResult.unit!.declaredElement!.source.fullName;

        // var globList = AnalysisOptions.fromYamlMap(_yamlMap).excludes!.map((e) => Glob(p.join(_sourceUri!, e))).toList();
        // if (globList.any((glob) => glob.matches(_fullName))) {
        //   return;
        // }

        var issueList = <Issue>[];
        for (final rule in getAllRules()) {
          issueList.addAll(rule.check(analysisResult));
        }

        if (issueList.isNotEmpty) {
          channel.sendNotification(
            plugin.AnalysisErrorsParams(
              analysisResult.path!,
              issueList
                  .map((issue) =>
                      codeIssueToAnalysisError(issue, analysisResult))
                  .toList(),
            ).toNotification(),
          );
        } else {
          channel.sendNotification(
              plugin.AnalysisErrorsParams(analysisResult.path!, [])
                  .toNotification());
        }
      } else {
        channel.sendNotification(
            plugin.AnalysisErrorsParams(analysisResult.path!, [])
                .toNotification());
      }
    } on Exception catch (e, stackTrace) {
      channel.sendNotification(
          plugin.PluginErrorParams(false, e.toString(), stackTrace.toString())
              .toNotification());
    }
  }

  // @override
  // Future<EditGetFixesResult> handleEditGetFixes(
  //     EditGetFixesParams parameters) async {
  //   var path = parameters.file;
  //   var request = await getFixesRequest(parameters);
  //   var generator = FixGenerator(getFixContributors(path));
  //   var result = await generator.generateFixesResponse(request);
  //   result.sendNotifications(channel);
  //   return result.result;
  // }

  @override
  Future<plugin.EditGetFixesResult> handleEditGetFixes(plugin.EditGetFixesParams parameters) async {
    try {
      var path = parameters.file;
      var offset = parameters.offset;
      var analysisResult = await getResolvedUnitResult(path);

      var issueNeedFix = locateIssueAtGivenOffset(offset, analysisResult);
      final errorFixes = <AnalysisErrorFixes>[];
      if (issueNeedFix != null) {
        errorFixes.addAll(codeIssueToAnalysisErrorFixes([issueNeedFix], analysisResult));
      }

      return plugin.EditGetFixesResult(errorFixes);
    } on Exception catch (e, stackTrace) {
      channel.sendNotification(plugin.PluginErrorParams(false, e.toString(), stackTrace.toString()).toNotification());

      return plugin.EditGetFixesResult([]);
    }
  }
}
