import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart';
import 'package:cat_analyzer_plugin/model/error_issue.dart';
import 'package:cat_analyzer_plugin/model/rule.dart';

class AvoidHardCodedUrlRule extends Rule {
  AvoidHardCodedUrlRule(String ruleId) : super(ruleId: ruleId);

  @override
  String? get code => ruleId;

  @override
  String get comment => 'extract string';

  @override
  String get correction => 'check for validation';

  // @override
  // String get message => 'do not use hard coded string in $methodName Widget';

  @override
  String get message => 'avoid using hard coded url';

  @override
  String get methodName => 'Text';

  @override
  Iterable<Issue> check(ResolvedUnitResult analysisResult) {
    final visitor =
    HardCodedStringIssueVisitor(analysisResult: analysisResult, rule: this);
    analysisResult.unit!.accept(visitor);
    return visitor.nodes
        .map((node) => Issue(
        errorSeverity: AnalysisErrorSeverity.INFO,
        errorType: AnalysisErrorType.HINT,
        offset: node.offset,
        length: node.length,
        line: analysisResult.unit!.lineInfo!
            .getLocation(node.offset)
            .lineNumber,
        column: analysisResult.unit!.lineInfo!
            .getLocation(node.offset)
            .columnNumber,
        endLine:
        analysisResult.unit!.lineInfo!.getLocation(node.end).lineNumber,
        endColumn: analysisResult.unit!.lineInfo!
            .getLocation(node.end)
            .columnNumber,
        message: message,
        code: code,
        correction: correction,
        hasFix: false,
        filePath: analysisResult.unit!.declaredElement!.source.fullName,
        comment: comment))
        .toList();
  }
}

class HardCodedStringIssueVisitor extends RecursiveAstVisitor<void> {
  final _nodes = <AstNode>[];
  final ResolvedUnitResult? analysisResult;
  final Rule? rule;

  HardCodedStringIssueVisitor({this.analysisResult, this.rule});

  Iterable<AstNode> get nodes => _nodes;

  /// e.g.
  /// String s = "s";
  @override
  void visitVariableDeclarationStatement(VariableDeclarationStatement node) {
    var hasIssue = false;
    var declarationList = node.variables.variables;
    if (declarationList.length == 1) {
      var declaration = declarationList.first;
      hasIssue = _checkUsingHardCodedString(declaration.initializer);
    }
    if (!hasIssue) {
      super.visitVariableDeclarationStatement(node);
    }
  }

  /// e.g.
  /// s = "s";
  @override
  void visitExpressionStatement(ExpressionStatement node) {
    var hasIssue = false;
    if (node.expression is AssignmentExpression) {
      hasIssue = _checkUsingHardCodedString(
          (node.expression as AssignmentExpression).rightHandSide);
    }
    if (!hasIssue) {
      super.visitExpressionStatement(node);
    }
  }

  // /// 创建对象的语义分析会走这个方法, e.g.
  // /// Widget text = Text("s");
  // @override
  // void visitInstanceCreationExpression(InstanceCreationExpression node) {
  //   var hasIssue = false;
  //   if (node.constructorName.type.name.name == 'Text') {
  //     hasIssue = _checkUsingHardCodedString(node.argumentList.arguments[0]);
  //   }
  //   if (!hasIssue) {
  //     super.visitInstanceCreationExpression(node);
  //   }
  // }
  //
  // /// 方法调用的语义分析会走这个方法, e.g.
  // /// Row(
  // ///   children: [
  // ///      Text("s")
  // ///   ]
  // /// )
  // ///
  // @override
  // void visitMethodInvocation(MethodInvocation node) {
  //   var hasIssue = false;
  //   if (node.methodName.name == 'Text') {
  //     hasIssue = _checkUsingHardCodedString(node.argumentList.arguments[0]);
  //   }
  //   if (!hasIssue) {
  //     super.visitMethodInvocation(node);
  //   }
  // }

  bool _checkUsingHardCodedString(Expression? expression) {
    if (expression is SimpleStringLiteral) {
      if (expression.value.isNotEmpty && checkIsUrl(expression.value)) {
        _nodes.add(expression);
        return true;
      }
    }
    return false;
  }
}

bool checkIsUrl(String text) {
  final urlRegex = RegExp(
      r'https?:\/\/(www\\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_\+.~#?&/=]*)',
      caseSensitive: false);
  final matchRet = urlRegex.allMatches(text);
  for (final match in matchRet) {
    final content = match.group(0);
    if (content != null) {
      // 存在匹配的字符串
      return true;
    }
  }
  return false;
}