import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart';
import 'package:cat_analyzer_plugin/model/error_issue.dart';
import 'package:cat_analyzer_plugin/model/rule.dart';
import 'package:_fe_analyzer_shared/src/scanner/token_impl.dart'
    show StringToken;

class PreferDoubleQuotesRule extends Rule {
  PreferDoubleQuotesRule(String ruleId) : super(ruleId: ruleId);

  @override
  String? get code => ruleId;

  @override
  String get comment => 'convert to double quotes string';

  @override
  String get correction => 'using double quotes';

  @override
  String get message => 'do not use Single quotes for String declaration';

  @override
  String? get methodName => 'PreferDoubleQuotesRule';

  @override
  Iterable<Issue> check(ResolvedUnitResult analysisResult) {
    final visitor =
    SingleQuotesIssueVisitor(analysisResult: analysisResult, rule: this);
    analysisResult.unit!.accept(visitor);
    return visitor.nodes
        .map((node) => Issue(
      errorSeverity: AnalysisErrorSeverity.INFO,
      errorType: AnalysisErrorType.HINT,
      offset: node.offset,
      length: node.length,
      line: analysisResult.unit!.lineInfo!
          .getLocation(node.offset)
          .lineNumber,
      column: analysisResult.unit!.lineInfo!
          .getLocation(node.offset)
          .columnNumber,
      endLine: analysisResult.unit!.lineInfo!
          .getLocation(node.end)
          .lineNumber,
      endColumn: analysisResult.unit!.lineInfo!
          .getLocation(node.end)
          .columnNumber,
      message: message,
      code: code,
      correction: correction,
      hasFix: false,
      filePath: analysisResult.unit!.declaredElement!.source.fullName,
      comment: comment
    ))
        .toList();
  }

}

class SingleQuotesIssueVisitor extends RecursiveAstVisitor<void> {
  final _nodes = <AstNode>[];
  final ResolvedUnitResult? analysisResult;
  final Rule? rule;

  SingleQuotesIssueVisitor({this.analysisResult, this.rule});

  Iterable<AstNode> get nodes => _nodes;

  @override
  void visitSimpleStringLiteral(SimpleStringLiteral node) {
    if (node.parent is ImportDirective) {
      return;
    }
    var hasIssue = false;
    if (node.literal is StringToken) {
      String contentWithQuote = (node.literal as StringToken).valueOrLazySubstring;
      if (contentWithQuote.startsWith("'") && contentWithQuote.endsWith("'")) {
        _nodes.add(node);
        hasIssue = true;
      }
    }
    if (!hasIssue) {
      super.visitSimpleStringLiteral(node);
    }
  }
}
